/**
 * @file
 * Contains definition of the behaviour Phone Number.
 */

(function ($, Drupal, drupalSettings, once) {
  "use strict";

  Drupal.behaviors.phoneNumber = {
    attach: function (context, settings) {
      // Get phone field settings.
      const options = drupalSettings.phone.options;

      once("phoneNumber", ".form-phone").forEach((field) => {
        // Get updated options for alter phone library.
        Drupal.behaviors.phoneNumber.alterPhoneLibraryOptions(field, options);
        // Initial phone library.
        Drupal.initPhoneNumber(field, options);
      });

    },

    /**
     * Allow to alter phone library options.
     *
     * @param {Object} field
     *   The field element.
     * @param {Object} options
     *   The list of options to init intlTelInput.
     *
     * @return void
     */
    alterPhoneLibraryOptions: (field, options) => {}
  };

  /**
   * Initial phone number by widget settings.
   *
   * @param {Object} field
   *   The field element.
   * @param {Object} options
   *   The list of options to init intlTelInput.
   */
  Drupal.initPhoneNumber = function (field, options, countryCode) {
    // Create the jQuery element that will be used in the initial steps.
    const $phoneMasked = $(field);
    const $phoneParent = $phoneMasked.closest('div.field--type-phone');
    const $phoneLabels = $phoneParent.find("label");
    const $phoneNumber = $phoneParent.find('[name*="phone_number"]');
    const $countryCode = $phoneParent.find('[name*="country_code"]');
    const $countryIso2 = $phoneParent.find('[name*="country_iso2"]');
    const $errorStatus = $phoneParent.find(".phone-error-msg");

    // Set initialCountry by country iso2 default value for existing number.
    if ($countryIso2.length !== 0 && $countryIso2.val() !== '') {
      options.initialCountry = $countryIso2.val();
    }

    // Provide geoIP lookup function.
    options.geoIpLookup = function(callback) {
      let remoteApiUrl = options.geoLocationApi.url;
      if (typeof options.geoLocationApi.apiKey !== 'undefined' && options.geoLocationApi.apiKey.length) {
        remoteApiUrl = remoteApiUrl + options.geoLocationApi.apiKey;
      }
      fetch(remoteApiUrl)
        .then(function(response) {
          switch (options.geoLocationApi.type) {
            case 'text':
              return response.text();
            case 'json':
              return response.json();
          }
        })
        .then(function(data) {
          let countryCode = new Function('data', options.geoLocationApi.script);
          callback(countryCode(data));
        })
        .catch(function() { callback("us"); });
    }

    // Additional options to expand the mask functionality and remove
    // the zero from beginning of local number for phone library.
    let removedZero = options.removeStartZero;
    let isFormatter = options.maskFormatter;

    // Reset function to reset error state on validation.
    const reset = function() {
      $phoneMasked.removeClass("has-error");
      $phoneLabels.removeClass("has-error");
      $errorStatus.addClass("hidden-xs-up");
    };

    // Init phone number plugin by widget settings.
    $phoneMasked.intlTelInput(options);

    // Phone widget wrapper.
    let $fieldWrapper = $phoneParent.find('div.iti');

    // Change hidden inputs provided in phone widget formElement.
    if ($phoneMasked && $fieldWrapper.length > 0) {
      // Set full number by phone number default value for existing number.
      if ($phoneNumber.length !== 0 && $phoneNumber.val() !== '') {
        field.nextElementSibling.value = $phoneNumber.val();
      }
      // Remove the hidden phone number input provided in the formElement
      // phone widget because it is re-added by the phone library.
      $phoneNumber.remove();
      // Move country code and iso2 elements into the phone widget wrapper.
      $countryCode.appendTo($fieldWrapper);
      $countryIso2.appendTo($fieldWrapper);
    }

    // Get placeholder attribute.
    let placeHolder = $phoneMasked.attr("placeholder");

    // Get selected country data for placeholder initial value.
    let iso2Data = $phoneMasked.intlTelInput("getSelectedCountryData").iso2;
    let dialCode = $phoneMasked.intlTelInput("getSelectedCountryData").dialCode;

    // Utils are lazy loaded, so must check.
    if (typeof intlTelInputUtils !== 'undefined' && typeof iso2Data !== 'undefined') {
      placeHolder = intlTelInputUtils.getExampleNumber(iso2Data, false, intlTelInputUtils.numberFormat.INTERNATIONAL);

      // Update placeholder initial value by selected country.
      let replace = isZeroStart ? '' : '0';
      placeHolder = placeHolder.replace("+" + dialCode, replace);
      $phoneMasked.attr("placeholder", placeHolder);
    }

    // Initial phone mask number formatter.
    if (isFormatter && typeof placeHolder !== 'undefined') {
      if (removedZero) {
        placeHolder = placeHolder.replace(/^0+/, '');
      }
      Drupal.maskPlaceholder($phoneMasked, placeHolder);
    }

    // On blur: validate.
    $phoneMasked.blur(function() {
      // Reset states.
      reset();

      if ($.trim($phoneMasked.val())) {
        // If number is not valid.
        if ($phoneMasked.intlTelInput("isValidNumber")) {
          // Set hidden input to dial code + inputted number.
          $phoneNumber.val($phoneMasked.intlTelInput("getSelectedCountryData").dialCode + $phoneMasked.val());
        }
        else {
          // Set error states.
          $phoneMasked.addClass("has-error");
          $phoneLabels.addClass("has-error");
          $errorStatus.removeClass("hidden-xs-up");
          // Clear hidden input val.
          $phoneNumber.val("");
        }
      }
    });

    // Reset on keyup / change flag.
    $phoneMasked.on("keyup change", reset);

    // Remove zero from beginning on keyup.
    if (removedZero) {
      field.addEventListener('keyup', function(e) {
        var input = this.value;
        if (input[0] == "0") {
          this.value = input.replace(/^0+/, "");
        }
        if (input.trim().length === 0 && removedZero) {
          let placeHolder = field.getAttribute("placeholder").replace(/^0+/, '');
          $phoneMasked.attr("placeholder", placeHolder);
        }
      });
    }

    // Country change event listener.
    field.addEventListener('countrychange', function() {
      // Update country code and iso2 values via plugin.
      $countryCode.val($phoneMasked.intlTelInput("getSelectedCountryData").dialCode);
      $countryIso2.val($phoneMasked.intlTelInput("getSelectedCountryData").iso2);

      // Get selected country placeholder via plugin example number format.
      let placeHolder = $phoneMasked.attr("placeholder");

      // Remove zero from beginning.
      if (removedZero) {
        placeHolder = placeHolder.replace(/^0+/, '');
        // Update placeholder to no leading zero.
        $phoneMasked.attr("placeholder", placeHolder);
      }

      // Update phone mask number formatter.
      if (isFormatter) {
        setTimeout(function () {
          // Use updated placeholder to define formatting pattern.
          Drupal.maskPlaceholder($phoneMasked, placeHolder);
        }, 120);
      }
    });
  };

  /**
   * Phone create mask from placeholder.
   */
  Drupal.maskPlaceholder = function ($phoneMasked, placeHolder) {
    let maskPattern = placeHolder.replace(new RegExp("[0-9]", "g"), "9").replace(/([9]\d{0,10})/g, '{{$1}}');

    // update formatter pattern.
    if ($phoneMasked.data('phone-mask')) {
      $phoneMasked.formatter().resetPattern(maskPattern);
    }
    else {
      // init mask formatter.
      $phoneMasked.formatter({
        'pattern': maskPattern
      });
      $phoneMasked.attr('data-phone-mask', true);
    }
  };

})(jQuery, Drupal, drupalSettings, once);
