<?php

namespace Drupal\phonenumber\Element;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Render\Element\FormElement;

/**
 * Provides a form element for entering an international phone number.
 *
 * Provides an HTML5 input element with type of "tel". It provides no special
 * validation.
 *
 * Properties:
 * - #phone
 *    - allowed_countries.
 *    - allowed_types.
 *    - placeholder.
 *    - extension_field.
 *    - phone_size.
 *    - extension_size.
 * - #size: The size of the input element in characters.
 * - #pattern: A string for the native HTML5 pattern attribute.
 *
 * Usage example:
 *
 * @code
 * $form['phone'] = [
 *   '#type' => 'phone',
 *   '#title' => $this->t('Phone'),
 *   '#pattern' => '[^\d]*',
 * ];
 * @endcode
 *
 * @see \Drupal\Core\Render\Element
 *
 * @FormElement("phone")
 */
class Phone extends FormElement {

  const PHONE_MAX_LENGTH = 16;

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = static::class;
    return [
      '#input' => TRUE,
      '#size' => 30,
      '#maxlength' => self::PHONE_MAX_LENGTH,
      '#autocomplete_route_name' => FALSE,
      '#process' => [
        [$class, 'processAutocomplete'],
        [$class, 'processAjaxForm'],
        [$class, 'processPattern'],
      ],
      '#element_validate' => [
        [$class, 'validatePhone'],
      ],
      '#pre_render' => [
        [$class, 'preRenderPhone'],
      ],
      '#theme' => 'input__phone',
      '#theme_wrappers' => ['form_element'],
      '#phone' => [],
    ];
  }

  /**
   * {@inheritdoc}
   */
  /*public static function valueCallback(&$element, $input, FormStateInterface $form_state) {
    if ($input) {
      $settings = !empty($element['#phone']) ? $element['#phone'] : [];
      $settings += [
        'exclude_countries' => [],
        'allowed_types' => NULL,
        'extension_field' => FALSE,
      ];

      $extension = $settings['extension_field'] ? $input['extension'] : NULL;
      $result = [
        'phone_number' => $input['phone_number'],
        'local_number' => $input['local_number'],
        'country_code' => $input['country_code'],
        'country_iso2' => $input['country_iso2'],
        'extension' => $extension,
      ];
    }
    else {
      $result = !empty($element['#default_value']) ? $element['#default_value'] : [];
    }

    return $result;
  }*/

  /**
   * Form element validation handler for #type 'phone'.
   *
   * Note that #maxlength and #required is validated by _form_validate() already.
   */
  public static function validatePhone(&$element, FormStateInterface $form_state, &$complete_form) {
   /* $value = trim($element['#value']);
    var_dump($element); exit;
    $form_state->setValueForElement($element, $value);

    if ($value !== '' && !\Drupal::service('email.validator')->isValid($value)) {
      $form_state->setError($element, t('The email address %mail is not valid.', ['%mail' => $value]));
    }*/
  }

  /**
   * Prepares a #type 'tel' render element for input.html.twig.
   *
   * @param array $element
   *   An associative array containing the properties of the element.
   *   Properties used: #title, #value, #description, #size, #maxlength,
   *   #placeholder, #required, #attributes.
   *
   * @return array
   *   The $element with prepared variables ready for input.html.twig.
   */
  public static function preRenderPhone($element) {
    $element['#attributes']['type'] = 'tel';
    Element::setAttributes($element, ['id', 'name', 'value', 'size', 'maxlength', 'placeholder']);
    static::setAttributes($element, ['form-phone']);

    // Get phone settings.
    $settings = $element['#phone'];

    // Get Module path to load utilsTellInput.js.
    $module_handler = \Drupal::service('module_handler');
    $module_path = base_path() . $module_handler->getModule('phonenumber')->getPath();;

    // Set utils script file url.
    $utils_script = $module_path . '/lib/js/utils.js';

    // Get GeoLocation API settings.
    $settings['geolocation_api'] = $settings['geolocation_api'] ?? 'ipapi';
    $geolocation = phonenumber_geo_ip_lookup_services();
    $geolocation_api = [
      'name' => $settings['geolocation_api'],
      'url' => $geolocation[$settings['geolocation_api']]['url'],
      'script' => $geolocation[$settings['geolocation_api']]['script'],
      'type' => $geolocation[$settings['geolocation_api']]['type'],
    ];

    // Add geolocation API key.
    if (isset($geolocation[$settings['geolocation_api']]['signup']) && $geolocation[$settings['geolocation_api']]['signup'] && isset($settings['geolocation_key']) && !empty(trim($settings['geolocation_key']))) {
      $geolocation_api['apiKey'] = trim($settings['geolocation_key']);
    }

    // Get phone mask number formatter.
    $mask_formatter = !isset($settings['mask_formatter']) || $settings['mask_formatter'];

    // Phone default country.
    $default_country = $settings['initial_country'] ?? 'auto';

    // Phone top list country.
    $preferred_countries = $element['#phone']['preferred_countries'] ?? ["us", "gb"];
    if (isset($settings['initial_country']) && $settings['initial_country'] != 'auto' && !in_array($settings['initial_country'], $preferred_countries)) {
      $preferred_countries = array_merge([$element['#country']], $preferred_countries);
    }

    // Provide library options from phone settings.
    $options = [
      // Whether to allow the dropdown.
      'allowDropdown'         => $settings['allow_dropdown'] ?? TRUE,
      // Auto insert dial code (A) on init, (B) on user selecting a country, (C) on calling setCountry
      // also listen for blur/submit and auto remove dial code if that's all there is
      'autoInsertDialCode'    => $settings['auto_insert_dial_code'] ?? FALSE,
      // Add a placeholder in the input with an example number for the selected country
      'autoPlaceholder'       => $settings['auto_placeholder'] ?? 'polite',
      // Modify the parentClass.
      'customContainer'       => $settings['custom_container'] ?? '',
      // Modify the auto placeholder.
      'customPlaceholder'     => $settings['custom_placeholder'] ?? '',
      // Append menu to specified element.
      'dropdownContainer'     => null,
      // Don't display these countries.
      'excludeCountries'      => $settings['exclude_countries'] ?? [],
      // format the input value during initialisation and on setNumber
      'formatOnDisplay'       => $settings['format_on_display'] ?? TRUE,
      // geoIp lookup function
      'geoIpLookup'           => $default_country === 'auto',
      // inject a hidden input with this name, and on submit, populate it with the result of getNumber
      'hiddenInput'           => "phone_number",
      // initial country
      'initialCountry'        => $default_country,
      // localized country names e.g. { 'de' => 'Deutschland' }
      'localizedCountries'    => $settings['localized_countries'] ?? [],
      // national vs international formatting for numbers e.g. placeholders and displaying existing numbers
      'nationalMode'          => $settings['national_mode'] ?? TRUE,
      // display only these countries
      'onlyCountries'         => $settings['only_countries'] ?? [],
      // number type to use for placeholders
      'placeholderNumberType' => $settings['placeholder_number_type'] ?? 'MOBILE',
      // the countries at the top of the list. defaults to the United States and United Kingdom
      'preferredCountries'    => $preferred_countries,
      // display the country dial code next to the selected flag
      'separateDialCode'      => $settings['separate_dial_code'] ?? FALSE,
      // option to hide the flags - must be used with separateDialCode, or allowDropdown=false
      'showFlags'             => $settings['show_flags'] ?? TRUE,
      // specify the path to the libphonenumber script to enable validation/formatting
      'utilsScript' => $utils_script,
      // Add mask and format based on country international phone number pattern.
      'maskFormatter' => $mask_formatter,
      // Remove zero from Placeholder beginning for local number.
      'removeStartZero' => $settings['remove_start_zero'] ?? TRUE,
      // IP geolocation API for lookup user's country code.
      'geoLocationApi' => $geolocation_api,
    ];

    // Attach phone options.
    $element['#attached']['drupalSettings']['phone']['options'] = $options;

    // Attach phone widget and number formatter library.
    if ($mask_formatter) {
      $element['#attached']['library'][] = 'phonenumber/phonenumber.formatter';
    }
    $element['#attached']['library'][] = 'phonenumber/phone';

    return $element;
  }

}
