<?php

namespace Drupal\phonenumber\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Plugin implementation of the 'phone' field type.
 *
 * @FieldType(
 *   id = "phone",
 *   label = @Translation("Phone number"),
 *   description = @Translation("This field stores and parses the phone number in international format."),
 *   default_widget = "phone_default",
 *   default_formatter = "basic_string"
 * )
 */
class PhoneItem extends FieldItemBase {

  /**
   * The maximum length for a phone value.
   */
  const MAX_LENGTH = 16;

  /**
   * {@inheritdoc}
   */
  public static function defaultStorageSettings() {
    return [
      'unique' => FALSE,
    ] + parent::defaultStorageSettings();
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultFieldSettings() {
    return [
      'national_mode' => TRUE,
      'countries' => 'all',
      'exclude_countries' => [],
      'include_countries' => [],
      'geo_ip_lookup' => 'ipapi',
      'api_key' => '',
      'custom_container' => '',
      'extension_field' => FALSE,
      'enabled_localisation' => FALSE,
      'localized_countries' => [],
    ] + parent::defaultFieldSettings();
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'phone_number' => [
          'type' => 'varchar',
          'length' => self::MAX_LENGTH,
        ],
        'local_number' => [
          'type' => 'varchar',
          'length' => 18,
        ],
        'country_code' => [
          'type' => 'varchar',
          'length' => 3,
        ],
        'country_iso2' => [
          'type' => 'varchar',
          'length' => 2,
        ],
      ],
      'indexes' => [
        'value' => ['phone_number'],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['phone_number'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('International phone number'))
      ->addConstraint('Length', ['max' => self::MAX_LENGTH])
      ->setRequired(TRUE);

    $properties['local_number'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Local Number'))
      ->addConstraint('Length', ['max' => 15])
      ->setRequired(false);

    $properties['country_code'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Country dial code'))
      ->addConstraint('Length', ['max' => 3])
      ->setRequired(false);

    $properties['country_iso2'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Country ISO alpha-2'))
      ->addConstraint('Length', ['max' => 2])
      ->setRequired(false);

    $properties['extension'] = DataDefinition::create('string')
      ->setLabel(t('Extension'))
      ->addConstraint('Length', ['max' => 40])
      ->setRequired(false);

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $value = $this->get('phone_number')->getValue();
    return $value === NULL || $value === '';
  }

  /**
   * Is phone number unique within the entity/field.
   *
   * @return bool|null
   *   TRUE for is unique, false otherwise. Null if phone number is not valid.
   */
  public function isUnique() {
    $entity = $this->getEntity();
    $field_name = $this->getFieldDefinition()->getName();

    $values = $this->getValue();
    $number = $values['phone_number'];
    if (!$number) {
      return NULL;
    }
    $entity_type_id = $entity->getEntityTypeId();
    $id_key = $entity->getEntityType()->getKey('id');
    $query = \Drupal::entityQuery($entity_type_id)
      // The id could be NULL, so we cast it to 0 in that case.
      ->condition($id_key, (int) $entity->id(), '<>')
      ->accessCheck(TRUE)
      ->condition($field_name, $number)
      ->range(0, 1)
      ->count();

    return !(bool) $query->execute();
  }

  /**
   * {@inheritdoc}
   */
  public function storageSettingsForm(array &$form, FormStateInterface $form_state, $has_data) {
    $settings = $this->getSettings();

    $element = [];

    $element['unique'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Unique number'),
      '#default_value' => $settings['unique'],
      '#description' => $this->t('Should phone numbers be unique within this field.'),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function fieldSettingsForm(array $form, FormStateInterface $form_state) {
    // Get base form from FileItem.
    $element = parent::fieldSettingsForm($form, $form_state);
    $settings = $this->getSettings();

    /** @var \Drupal\Core\Locale\CountryManagerInterface $country_manager */
    $country_manager = \Drupal::service('country_manager');

    $countries = $country_manager->getList();
    $element['national_mode'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('National mode'),
      '#description' => $this->t('Disable national mode if you want to enter phone number with country code instead of display the country flag, then disable the "Show flag" and the "Allow dropdown" in manage form display.'),
      '#default_value' => $settings['national_mode'],
    ];
    $element['national_format'] = [
      '#type' => 'item',
      '#description' => $this->t('Format numbers in the national format (without need country code), rather than the international format (with country code) when the country can be selected. This applies to placeholder numbers, and when displaying existing numbers. It is recommended to leave this option enabled, to encourage to enter numbers in national format as this is usually more familiar and so it creates a better user experience.'),
      '#states' => [
        'visible' => [
          ':input[name="settings[national_mode]"]' => ['checked' => FALSE],
        ],
      ],
    ];

    $element['countries'] = [
      '#type' => 'radios',
      '#title' => $this->t('List of countries'),
      '#options' => [
        'all' => $this->t('All countries'),
        'include' => $this->t('Only the selected countries'),
        'exclude' => $this->t('Exclude the selected countries'),
      ],
      '#default_value' => $settings['countries'],
    ];

    $element['include_countries'] = [
      '#type' => 'select',
      '#title' => $this->t('Countries'),
      '#title_display' => 'invisible',
      '#options' => $countries,
      '#multiple' => TRUE,
      '#default_value' => $settings['exclude_countries'],
      '#states' => [
        'visible' => [
          '[name="settings[countries]"]' => ['value' => 'include'],
        ],
      ],
    ];

    $element['exclude_countries'] = [
      '#type' => 'select',
      '#title' => $this->t('Countries'),
      '#title_display' => 'invisible',
      '#options' => $countries,
      '#multiple' => TRUE,
      '#default_value' => $settings['exclude_countries'],
      '#states' => [
        'visible' => [
          '[name="settings[countries]"]' => ['value' => 'exclude'],
        ],
      ],
    ];

    $geolocation_api = phonenumber_geo_ip_lookup_services();
    $geolocation_titles = array_column($geolocation_api, 'title');
    $geo_ip_lookup = array_combine(array_keys($geolocation_api), $geolocation_titles);
    $element['geo_ip_lookup'] = [
      '#type' => 'select',
      '#title' => $this->t('IP Geolocation API service'),
      '#options' => $geo_ip_lookup,
      '#default_value' => $settings['geo_ip_lookup'],
      '#description' => $this->t("When setting <strong>Default country</strong> to <em>auto</em>, you must use this option to specify an IP Geolocation API service that looks up the user's location, and then callback with the relevant country code. Leave it to default, if you want to set the default country to a specific country."),
    ];

    $signup = array_column($geolocation_api, 'signup');
    $api_key_services = array_filter(array_combine(array_keys($geolocation_api), $signup));
    $api_key_states = [];
    foreach ($api_key_services as $service => $value) {
      $api_key_states['visible'][]['[name="settings[geo_ip_lookup]"]'] = ['value' => $service];
      $api_key_states['required'][]['[name="settings[geo_ip_lookup]"]'] = ['value' => $service];
    }
    $element['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API key'),
      '#default_value' => $this->getSetting('api_key'),
      '#description' => $this->t('Additional classes to add to the phone parent div. Separate each class with a space.'),
      '#states' => $api_key_states,
    ];

    $element['custom_container'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom container class'),
      '#default_value' => $this->getSetting('custom_container'),
      '#description' => $this->t('Additional classes to add to the phone parent div. Separate each class with a space.'),
    ];

    $element['enabled_localisation'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Localize country names'),
      '#description' => $this->t('Allow localisation of country names.'),
      '#default_value' => $settings['enabled_localisation'],
    ];

    $localized_countried = 'fr: "Frankreich", de: "Deutschland", es: "Spanien", it: "Italien", ch: "Schweiz", nl: "Niederlande", at: "Österreich", dk: "Dänemark",';
    $element['localized_countries'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Localized countries'),
      '#default_value' => $this->getSetting('localized_countries'),
      '#description' => $this->t('Enter one country per line and for each country, the key should be the iso2 country code, and the value should be the localised country name. e.g. dk: "Dänemark".'),
      '#rows' => '6',
      '#states' => [
        'visible' => [
          ':input[name="settings[enabled_localisation]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $element['extension_field'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable extension field'),
      '#description' => $this->t('Collect extension along with the phone number.'),
      '#default_value' => $settings['extension_field'],
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function getConstraints() {
    $constraint_manager = \Drupal::typedDataManager()->getValidationConstraintManager();

    $constraints[] = $constraint_manager->create('ComplexData', [
      'phone_number' => [
        'Length' => [
          'max' => self::MAX_LENGTH,
          'maxMessage' => $this->t('%name: the phone number may not be longer than @max characters.', ['%name' => $this->getFieldDefinition()->getLabel(), '@max' => self::MAX_LENGTH]),
        ],
      ],
    ]);

    return $constraints;
  }

  /**
   * Get phone number object of the current item.
   *
   * @param bool $throw_exception
   *   Whether to throw phone number validity exceptions.
   *
   * @return \libphonenumber\PhoneNumber|null
   *   Phone number object, or null if not valid.
   */
  public function getPhoneNumber($throw_exception = FALSE) {
    $values = $this->getValue();
    $number = $values['phone_number'];
    /*$number = '';
    $country = NULL;
    $extension = NULL;
    if (!empty($values['country'])) {
      if (!empty($values['local_number'])) {
        $number = $values['local_number'];
      }
      $country = $values['country'];
    }

    if (!$number && !empty($values['value'])) {
      $number = $values['value'];
    }

    if (!empty($values['extension'])) {
      $extension = $values['extension'];
    }

    if ($throw_exception) {
      return $util->testPhoneNumber($number, $country, $extension);
    }
    else {
      return $util->getPhoneNumber($number, $country, $extension);
    }*/

  }

  /**
   * {@inheritdoc}
   */
  /*public function preSave() {
    parent::preSave();
    //var_dump($this->full); exit;
    $name = $this->getFieldDefinition()->getName();
    $values = $this->getValue();
  }*/

  /**
   * {@inheritdoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition) {
    $values['phone_number'] = rand(pow(10, 8), pow(10, 9) - 1);
    return $values;
  }

}
