<?php

namespace Drupal\phonenumber\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Locale\CountryManagerInterface;
use Drupal\phonenumber\Plugin\Field\FieldType\PhoneItem;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'phone_default' widget.
 *
 * @FieldWidget(
 *   id = "phone_default",
 *   label = @Translation("Phone number"),
 *   field_types = {
 *     "phone",
 *     "telephone"
 *   }
 * )
 */
class PhoneDefaultWidget extends WidgetBase {

  /**
   * The country manager.
   *
   * @var \Drupal\Core\Locale\CountryManagerInterface
   */
  protected CountryManagerInterface $countryManager;

  /**
   * Constructs a RegionalForm object.
   *
   * @param string $plugin_id
   *   The plugin_id for the widget.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the widget is associated.
   * @param array $settings
   *   The widget settings.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\Core\Locale\CountryManagerInterface $country_manager
   *   The country manager.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, CountryManagerInterface $country_manager) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);

    $this->countryManager = $country_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('country_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'initial_country' => 'auto',
      'preferred_countries' => ['US', 'GB'],
      'allow_dropdown' => TRUE,
      'auto_insert_dial_code' => FALSE,
      'separate_dial_code' => FALSE,
      'format_on_display' => TRUE,
      'show_flags' => TRUE,
      'use_fullscreen_popup' => TRUE,
      'remove_start_zero' => TRUE,
      'mask_formatter' => TRUE,
      'auto_placeholder' => 'polite',
      'placeholder_number_type' => 'MOBILE',
      'custom_placeholder' => '',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);
    $field_settings = $this->getFieldSettings();
    $allowed_countries = $this->countryManager->getList();
    $preferred_countries = $this->getSetting('preferred_countries');
    $initial_country = $this->getSetting('initial_country');

    if ($field_settings['countries'] == 'include' && is_array($field_settings['include_countries']) && count($field_settings['include_countries'])) {
      $allowed_countries = array_intersect_key($allowed_countries, $field_settings['include_countries']);
      $preferred_countries = in_array($preferred_countries, $allowed_countries) ? $preferred_countries : array_diff_key($preferred_countries, $field_settings['include_countries']);
    }

    if ($field_settings['countries'] == 'exclude' && is_array($field_settings['exclude_countries']) && count($field_settings['exclude_countries'])) {
      $allowed_countries = array_diff_key($allowed_countries, $field_settings['exclude_countries']);
      $preferred_countries = in_array($preferred_countries, $allowed_countries) ? $preferred_countries : array_diff_key($preferred_countries, $field_settings['exclude_countries']);
    }

    $initial_country = in_array($initial_country, $allowed_countries) ? $initial_country : 'auto';
    $element['initial_country'] = [
      '#type' => 'select',
      '#title' => $this->t('Default country'),
      '#options' => ['auto' => $this->t("Auto user's country (geoIPLookup)")] + $allowed_countries,
      '#default_value' => $initial_country,
      '#description' => $this->t("Specify a default selection country. You can also set it to <em>Auto</em>, which will look up the user's country based on their IP address."),
      '#states' => [
        'invisible' => [
          '[name="fields[' . $this->fieldDefinition->getName() . '][settings_edit_form][settings][geolocation]"]' => [
            'checked' => TRUE,
          ],
        ],
      ],
    ];

    $element['preferred_countries'] = [
      '#type' => 'select',
      '#title' => $this->t('Top list countries'),
      '#multiple' => TRUE,
      '#options' => $allowed_countries,
      '#default_value' => $preferred_countries,
      '#description' => $this->t('Specify the countries to appear at the top of the list. If none is selected, it will follow alphabetical order.'),
    ];

    $element['allow_dropdown'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Allow dropdown'),
      '#default_value' => $this->getSetting('allow_dropdown'),
      '#description' => $this->t('If disabled, there is no dropdown arrow and the selected flag is not clickable. Display the selected flag on the right because it is just a status indicator.'),
    ];

    $element['auto_insert_dial_code'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Auto insert dial code'),
      '#default_value' => $this->getSetting('auto_insert_dial_code'),
      '#description' => $this->t('When enabled, the international dial code will be automatically inserted into the input.'),
    ];

    $element['separate_dial_code'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Separate dial code'),
      '#default_value' => $this->getSetting('separate_dial_code'),
      '#description' => $this->t('Display the country dial code next to the selected flag.'),
    ];

    $element['show_flags'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show flags'),
      '#default_value' => $this->getSetting('show_flags'),
      '#description' => $this->t('Disable to hide the flags e.g. for political reasons. Must be used in combination with "Separate dial code" option, or with setting "Allow dropdown" to disable.'),
    ];

    $element['format_on_display'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Format on display'),
      '#default_value' => $this->getSetting('format_on_display'),
      '#description' => $this->t('Format the input value (according to the nationalMode option) during initialisation.'),
    ];

    $element['remove_start_zero'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Remove start zero'),
      '#default_value' => $this->getSetting('remove_start_zero'),
      '#description' => $this->t('Remove zero from input and placeholder beginning for local number.'),
    ];

    $element['mask_formatter'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Mask formatter'),
      '#default_value' => $this->getSetting('mask_formatter'),
      '#description' => $this->t('Enable to add mask and format based on country international phone number pattern. Force to enter the phone number only according to the pattern and prevent entering letters and other special characters.'),
    ];

    $element['placeholder_number_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Placeholder number type'),
      '#default_value' => $this->getSetting('placeholder_number_type'),
      '#description' => $this->t('Enable to add mask and format based on country international phone number pattern. Force to enter the phone number only according to the pattern and prevent entering letters and other special characters.'),
      '#options' => phonenumber_number_types(),
    ];

    $element['custom_placeholder'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom placeholder'),
      '#default_value' => $this->getSetting('custom_placeholder'),
      '#description' => $this->t('Leave blank to display sample numbers for each country. Text that will be shown inside the field until a value is entered. If set a value or description, it will be displayed in general for all countries.'),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    $placeholder = $this->getSetting('placeholder');
    if (!empty($placeholder)) {
      $summary[] = $this->t('Placeholder: @placeholder', ['@placeholder' => $placeholder]);
    }
    else {
      $summary[] = $this->t('No placeholder');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $item = $items[$delta];
    $phone_number = $item->local_number ? preg_replace('/\s+/', '', $item->phone_mask) : '';

    /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    $entity = $items->getEntity();
    $settings = $this->getSettings() + static::defaultSettings();
    $settings = array_merge($this->getFieldSettings(), $settings);

    /*$default_country = empty($settings['allowed_countries']) ?
      $settings['default_country'] :
      (empty($settings['allowed_countries'][$settings['default_country']]) ?
        key($settings['allowed_countries']) : $settings['default_country']);*/

    $element['local_number'] = $element + [
      '#type' => 'phone',
      '#description' => $element['#description'],
      '#phone' => [
        'allow_dropdown' => (bool) $settings['allow_dropdown'] ?? TRUE,
        'auto_insert_dial_code' => (bool) $settings['auto_insert_dial_code'] ?? FALSE,
        'auto_placeholder' => $settings['auto_placeholder'] ?? 'polite',
        'custom_container' => $settings['custom_container'] ?? '',
        'custom_placeholder' => $settings['custom_placeholder'] ?? NULL,
        'format_on_display' => (bool) $settings['format_on_display'] ?? TRUE,
        'initial_country' => $settings['initial_country'] ?? 'auto',
        'preferred_countries' => $settings['preferred_countries'] ?? [],
        'countries' => $settings['countries'] ?? 'all',
        'only_countries' => $settings['countries'] === 'include' ? $settings['include_countries'] : NULL,
        'exclude_countries' => $settings['countries'] === 'exclude' ? $settings['exclude_countries'] : NULL,
        'localized_countries' => $settings['enabled_localisation'] && count($settings['localized_countries']) ? $settings['localized_countries'] : [],
        'national_mode' => (bool) $settings['national_mode'] ?? TRUE,
        'placeholder_number_type' => $settings['placeholder_number_type'] ?? 'MOBILE',
        'separate_dial_code' => (bool) $settings['separate_dial_code'] ?? FALSE,
        'show_flags' => (bool) $settings['show_flags'] ?? TRUE,
        'use_fullscreen_popup' => (bool) $settings['use_fullscreen_popup'] ?? TRUE,
        'remove_start_zero' => (bool) $settings['remove_start_zero'] ?? TRUE,
        'mask_formatter' => (bool) $settings['mask_formatter'] ?? TRUE,
        'geolocation_api' => $settings['geo_ip_lookup'] ?? 'ipapi',
        'geolocation_key' => $settings['api_key'] ?? '',
        'token_data' => !empty($entity) ? [$entity->getEntityTypeId() => $entity] : [],
      ],
      '#default_value' => $phone_number ?? NULL,
      '#maxlength' => PhoneItem::MAX_LENGTH,
      '#field_suffix' => '<div class="phone-error-msg">Invalid phone number.</div>'
    ];
    if ($settings['extension_field']) {
      $element['extension'] = [
        '#type' => 'textfield',
        '#default_value' => !empty($value['extension']) ? $value['extension'] : NULL,
        '#title' => $this->t('Extension'),
        '#title_display' => 'invisible',
        '#size' => $settings['extension_size'] ?? 5,
        '#maxlength' => 40,
        '#attributes' => [
          'class' => ['extension'],
          'placeholder' => $this->t('Ext.'),
        ],
      ];
    }
    $element['phone_number'] = [
      '#type' => 'hidden',
      '#default_value' => $item->phone_number ?? NULL,
      '#attributes' => ['class' => 'temporary-phone']
    ];
    $element['country_code'] = [
      '#type' => 'hidden',
      '#default_value' => $item->country_code ?? NULL,
      '#attributes' => ['class' => 'temporary-phone']
    ];
    $element['country_iso2'] = [
      '#type' => 'hidden',
      '#default_value' => $item->country_iso2 ?? NULL,
      '#attributes' => ['class' => 'temporary-phone']
    ];

    return $element;
  }

}
